<?php

namespace Transferito\Controllers;

use Transferito\Models\Transfer\CodeBase;
use Transferito\Models\Transfer\Database;
use Transferito\Models\Transfer\Archive;
use Transferito\Models\Core\Api as TransferitoAPI;

class Transfer {

    private $codeBase;
    private $dataBase;
    private $archive;
    private $api;

    public function __construct()
    {
        if(current_user_can('activate_plugins')) {
	        $this->api = new TransferitoAPI();
	        $this->codeBase = new CodeBase();
            $this->dataBase = new Database();
            $this->archive = new Archive();

            add_action("wp_ajax_create_transfer", array($this, "createTransfer"));
            add_action("wp_ajax_preparing_transfer", array($this, "prepareDownload"));
            add_action("wp_ajax_send_transfer_files", array($this, "transferFiles"));
            add_action("wp_ajax_clean_up_files", array($this, "cleanUp"));
            add_action("wp_ajax_status_check", array($this, "statusCheck"));
        }
    }

	public function statusCheck()
	{
		$response = $this->api->getStatus(sanitize_text_field($_GET['token']));
		$responseMessage = $response["message"];

		/**
		 *
		 */
		if ($response["code"] === 200) {
			wp_send_json_success(array(
				"completed" => (isset($responseMessage->response)) ? $responseMessage->response === "completed" : false
			));
		} else {
			wp_send_json_error("T", $response["code"]);
		}

    }

	public function createTransfer()
	{
		check_ajax_referer("create_wp_transfer", "security");

		/**
		 * Send the data to the API
		 */
		$response = $this->api->create($_POST["options"]);

		/**
		 * Check the folder array exists
		 * If it does set the folder paths
		 */
		if (isset($_POST["options"]["transfer_selected_folders"]) && isset($_POST["options"]["folder_path"]) && (count($_POST["options"]["folder_path"]) > 0)) {
			set_transient('transferito_folder_array', $_POST["options"]["folder_path"], 12 * HOUR_IN_SECONDS);
		}

		/**
		 * Check the exclude flag is set
		 * If it is set flag
		 */
		if (isset($_POST["options"]["exclude_transferito_plugin"])) {
			set_transient('transferito_exclude_plugin', TRANSFERITO_PATH, 12 * HOUR_IN_SECONDS);
		}

		/**
		 *
		 */
		if ($response["code"] === 200) {
			$result = $response["message"];

			$destinationURL = (isset($result->destinationUrl)) ? $result->destinationUrl : "";
			$transferId = (isset($result->transferId)) ? $result->transferId : 0;
			$transferToken = (isset($result->transferToken)) ? $result->transferToken : "";

			/**
			 * Save destination url
			 */
			set_transient('transferito_destination_url', $destinationURL, 12 * HOUR_IN_SECONDS);


			wp_send_json_success(array(
				"token"         => $transferToken,
				"transferId"    => $transferId,
				"hasSelected"   => isset($_POST["options"]["transfer_selected_folders"])
			));
		} else {
			wp_send_json_error($response["message"], $response["code"]);
		}
    }

    public function prepareDownload()
    {
    	try {
		    check_ajax_referer("transfer_token_start", "security");

		    $token = sanitize_text_field($_POST["token"]);

		    /**
		     * Check the token exists
		     */
		    if (!isset($token) || $token === "") {
			    $error = "Token was not specified";
			    wp_send_json_error($error, 400);
		    };

		    /**
		     * Get Transient
		     */
		    $folderPaths = get_transient('transferito_folder_array');
		    $excludeTransferitoPlugin = get_transient('transferito_exclude_plugin');
		    $destinationURL = get_transient('transferito_destination_url');
		    $selectedFolderEnabled = ($folderPaths !== false);
		    delete_transient('transferito_folder_array');
		    delete_transient('transferito_exclude_plugin');
		    delete_transient('transferito_destination_url');


		    /**
		     * Create transients for the paths
		     */
		    set_transient('transferito_file_paths', array(), 60 * 60 * 12);

		    /**
		     * Download files
		     */
		    $downloadedPath = $this->codeBase->download($selectedFolderEnabled, $folderPaths, $excludeTransferitoPlugin);
		    $sqlPath = $this->dataBase->download($excludeTransferitoPlugin, $destinationURL);

		    /**
		     * Set transient with transfer detail
		     */
		    set_transient('transferito_transfer_detail', array(
			    "token"     => $token,
			    "dbPath"    => $sqlPath,
			    "codePath"  => $downloadedPath
		    ));

		    /**
		     * Send response
		     */
		    wp_send_json_success(array(
			    "message"   => "Your backup has been created successfully",
			    "nonce"     => wp_create_nonce("send_transfer_files")
		    ));
	    } catch (\Exception $exception) {
		    wp_send_json_error("There was an error preparing yor transfer", 400);
	    }

    }

    public function transferFiles()
    {
        check_ajax_referer("send_transfer_files", "security");

        $transferDetail = get_transient('transferito_transfer_detail');
        delete_transient('transferito_transfer_detail');

        /**
         * Send files to API
         * @todo change to use api
         */
        $uploadedFiles = $this->api->files(array(
            "codebase"  => $transferDetail["codePath"],
            "database"  => $transferDetail["dbPath"]
        ), $transferDetail["token"]);

        /**
         * Create variables
         */
        $statusCode = $uploadedFiles["code"];
        $message = sanitize_text_field($uploadedFiles["message"]);
        $response = array(
            "message"   => $message
        );

        /**
         * Return reponses based on status
         */
        if ($statusCode === 200) {
            wp_send_json_success($response);
        } else {
            wp_send_json_error($response, $statusCode);
        }
    }

    public function cleanUp()
    {
        $pathArray = get_transient('transferito_file_paths');

        /**
         * Remove all transients
         */
        delete_transient('transferito_file_paths');

        /**
         * Remove codebase
         */
        $this->archive->removeUsedArchives($pathArray);

        /**
         * return response
         */
        $response = array(
            "message"   => "Your backup has been removed"
        );

        wp_send_json_success($response);
    }


}