<?php

namespace Transferito\Models\Settings;

use Transferito\Models\Core\Api as TransferitoAPI;
use Transferito\Models\Core\Config;

class Setup {

    /**
     * Holds the values to be used in the fields callbacks
     */
    private $options;
    private $emptyApiKeys;
    private $api;

    public function __construct()
    {
        if (current_user_can('activate_plugins')) {
            add_action( 'admin_menu', array($this, "createMenuPage"));
            add_action( 'admin_init', array($this, "createOptions"));
            add_action( 'admin_enqueue_scripts', array($this, "loadTransferitoAssets"));
	        add_action( 'admin_bar_menu', array($this, "addToWPAdminMenu"), 100);

	        $this->api = new TransferitoAPI();
	        $this->options = get_option( 'transferito_settings_option' );
	        $this->emptyApiKeys = !($this->options['public_transferito_key'] && $this->options['secret_transferito_key']);
        }
    }

    public function addToWPAdminMenu($adminBar)
    {
	    if (!is_admin()) {
	    	return false;
	    }

	    $planInformation = $this->api->planInformation();
	    if ($planInformation['code'] === 200) {
		    $transferLimit = (isset($planInformation['message']->transferLimit))
			    ? $planInformation['message']->transferLimit
			    : 0;

		    /**
		     * Logic to create the correct verbage
		     * For the tooltip
		     */
		    if ($transferLimit === 0) {
		    	$transferMessage = '0 available transfers';
		    } else {
			    $transferMessage = ($transferLimit > 2)
				    ? "{$transferLimit} available transfers"
				    : "${transferLimit} available transfer";
		    }

		    /**
		     * Add menu to the wp admin bar
		     */
		    $adminBar->add_menu( array(
			    'id'    => 'transferito-admin-bar',
			    'title' => $transferLimit,
			    'href'  => admin_url('admin.php?page=transferito-main'),
			    'meta'  => array(
				    'title' => __($transferMessage),
			    ),
		    ));
	    }

    }

    public function createMenuPage()
    {
	    add_menu_page(
            __( 'Transferito1', 'textdomain' ),
            'Transferito',
            'manage_options',
            'transferito-main',
            '',
            'none',
            26
        );

	    add_submenu_page(
		    'transferito-main',
		    'Start Transfer',
		    'Start Transfer',
		    'manage_options',
		    'transferito-main',
		    array($this, 'transferHTML')
	    );


	    add_submenu_page(
		    'transferito-main',
		    'Create Transfer',
		    'Create Transfer',
		    'manage_options',
		    'transferito-create-transfer',
		    array($this, 'createTransferHTML')
	    );

        add_submenu_page(
	        'transferito-main',
            'Settings',
            'Settings',
            'manage_options',
            'transferito-settings',
            array($this, 'settingsHTML')
        );


    }

    public function createOptions()
    {
        register_setting(
            'transferito_settings_group', // Option group
            'transferito_settings_option', // Option name
            array( $this, 'sanitize' ) // Sanitize
        );

        add_settings_section(
            'transferito_settings_section', // ID
            '', // Title
            array( $this, 'settingSection' ), // Callback
            'transferito-settings' // Page
        );

        /**
         * API Keys fields
         */
        add_settings_field(
            'public_transferito_key', // ID
            'Public Key', // Title
            array( $this, 'publicKeyField' ), // Callback
            'transferito-settings', // Page
            'transferito_settings_section'
        );

        add_settings_field(
            'secret_transferito_key', // ID
            'Secret Key', // Title
            array( $this, 'secretKeyField' ), // Callback
            'transferito-settings', // Page
            'transferito_settings_section'
        );
    }

    public function loadTransferitoAssets($hook)
    {
        $script = plugins_url( '../Views/Assets/transferito.js', dirname(__FILE__));
        $style = plugins_url( '../Views/Assets/transferito.css', dirname(__FILE__));
        $dashIconFont = plugins_url( '../Views/Assets/transferito-font.css', dirname(__FILE__));

        wp_register_style( 'transferito_css', $style, false, '1.0.0' );
        wp_register_style( 'transferito_font_css', $dashIconFont, false, '1.0.0' );
        wp_register_script('transferito_js',
            $script,
            array ('jquery'),
            false, false);

        wp_enqueue_style( 'transferito_css' );
        wp_enqueue_style( 'transferito_font_css' );

//        if ($hook === 'transferito_page_transferito-transfer') {
            wp_enqueue_script( 'transferito_js' );
//        }
    }

    public function sanitize( $input )
    {
        $new_input = array();

        if( isset( $input['public_transferito_key'] ) )
            $new_input['public_transferito_key'] = sanitize_text_field( $input['public_transferito_key'] );

        if( isset( $input['secret_transferito_key'] ) )
            $new_input['secret_transferito_key'] = sanitize_text_field( $input['secret_transferito_key'] );

        return $new_input;
    }

    /**
     * Print the Section text
     */
    public function settingSection()
    {
        print '';
    }


    /**
     * Get the settings option array and print one of its values
     */
    public function publicKeyField()
    {
        printf(
            '<input type="text" class="transferito-input" id="public_transferito_key" name="transferito_settings_option[public_transferito_key]" value="%s" />',
            isset( $this->options['public_transferito_key'] ) ? esc_attr( $this->options['public_transferito_key']) : ''
        );
    }

    public function secretKeyField()
    {
        printf(
            '<input type="text" class="transferito-input" id="secret_transferito_key" name="transferito_settings_option[secret_transferito_key]" value="%s" />',
            isset( $this->options['secret_transferito_key'] ) ? esc_attr( $this->options['secret_transferito_key']) : ''
        );
    }

    public function settingsHTML()
    {
        echo loadTemplate("settings", array(
        	'publicKey'    => $this->options['public_transferito_key'],
	        'secretKey'     => $this->options['secret_transferito_key'],
	        'emptyAPIKeys'  => $this->emptyApiKeys
        ));
    }

    public function transferHTML()
    {
	    echo loadTemplate("transfer", array(
	        'emptyAPIKeys'  => $this->emptyApiKeys
        ));
    }

    public function createTransferHTML()
    {
	    /**
	     * Base template data
	     */
	    $templateData = array(
	    	'emptyAPIKeys'  => $this->emptyApiKeys
	    );

	    /**
	     * Pass in the correct template array data
	     * If the user has api keys
	     */
	    if (!$this->emptyApiKeys) {
		    $templateData = array_merge(
		    	$templateData,
			    array(
				    'emptyAPIKeys'  => $this->emptyApiKeys,
				    'basePath'      => WP_CONTENT_DIR,
				    'directories'   => Config::getWPContentPaths()
		        ),
			    $this->api->getSavedDetail()
		    );
	    }
	    echo loadTemplate("create-transfer", $templateData);
    }

}